# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kindwise', 'kindwise.tests']

package_data = \
{'': ['*'],
 'kindwise': ['resources/*'],
 'kindwise.tests': ['resources/images/*']}

install_requires = \
['pillow', 'python-dotenv', 'requests']

setup_kwargs = {
    'name': 'kindwise',
    'version': '0.1.2',
    'description': 'Python SDK toolkit for integrating Kindwise API',
    'long_description': '# Kindwise sdk for python\n\nPython SDK toolkit for integrating Kindwise API into your application. This Python SDK provides a convenient way to\ninteract with the Kindwise API for plant, insect, and mushroom identification. The SDK is organized into different\nmodules, each corresponding to a specific domain ([plant](https://web.plant.id/plant-identification-api/),\n[insect](https://www.kindwise.com/insect-id), [mushroom](https://www.kindwise.com/mushroom-id)). You can always use our\nAPI without our SDK, the documentation can be found on the following links:\n\n- [plant](https://documenter.getpostman.com/view/24599534/2s93z5A4v2)\n- [insect](https://documenter.getpostman.com/view/3802128/2s93sZ5YeU)\n- [mushroom](https://documenter.getpostman.com/view/3802128/2s93kz55EY)\n\n## Setup\n\n### Install\n\n```bash\npip install kindwise\n```\n\n### API key\n\nThe API key serves to identify your account and is required to make requests to the API. You can get your API key at\nadmin.kindwise.com. You need to register and create a new API key.\n\n## Quick Start\n\nTo use Kindwise API, you need to have an active API key. If you already do not have any, see the API key section.\n\n```python\nfrom kindwise.plant import PlantApi\nfrom kindwise.models import PlantIdentification, UsageInfo\n\n# initialize plant.id api\n# you do not have to specify api key here, if you have "PLANT_API_KEY" environment variable set.\napi = PlantApi(api_key=\'your_api_key\')\n\n# get usage information\nusage: UsageInfo = api.usage_info()\n\n# identify plant by image\nlatitude_longitude = (49.20340, 16.57318)\n# you must pass the image as a path\nimage_path = \'path/to/plant_image.jpg\'\n# make identification\nidentification: PlantIdentification = api.identify(image_path, latitude_longitude=latitude_longitude)\n\n# get identification by a token with changed views\n# this method can be used to modify additional information in identification or to get identification from database\n# also works with identification.custom_id\nidentification_with_different_views: PlantIdentification = api.get_identification(identification.access_token)\n\n# delete identification\napi.delete_identification(identification)  # also works with identification.access_token or identification.custom_id\n```\n\n## Structure\n\nSDK supports the following Kindwise systems:\n\n- [plant.id](https://web.plant.id/plant-identification-api/)\n- [insect.id](https://www.kindwise.com/insect-id)\n- [mushroom.id](https://www.kindwise.com/mushroom-id)\n\nEach system has its class, which is used to make requests to the API. Each class has the following methods:\n\n| method                                                    | description                                                                         | return type        | plant              | insect             | mushroom           |\n|-----------------------------------------------------------|-------------------------------------------------------------------------------------|--------------------|--------------------|--------------------|--------------------|\n| [`identify`](#identify)                                   | create new identification                                                           | `Identification`   | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`get_identification`](#get_identification)               | get identification by token                                                         | `Identification`   | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`delete_identification`](#delete_identification)         | delete identification by token                                                      | boolean            | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`usage_info`](#usage_info)                               | get api key usage information                                                       | `UsageInfo`        | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`feedback`](#feedback)                                   | send feedback for identification                                                    | boolean            | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`health_assessment`](#health_assessment)                 | create health assessment identification                                             | `HealthAssessment` | :white_check_mark: | :x:                | :x:                |\n| [`get_health_assessment`](#get_health_assessment)         | get health assessment identification                                                | `HealthAssessment` | :white_check_mark: | :x:                | :x:                |\n| [`delete_health_assessment`](#delete_health_assessment)   | delete health assessment                                                            | boolean            | :white_check_mark: | :x:                | :x:                |\n| [`available_details`](#available_details)                 | details which can be used to specify additional information for `identify`          | dict               | :white_check_mark: | :white_check_mark: | :white_check_mark: |\n| [`available_disease_details`](#available_disease_details) | details which can be used to specify additional information for `health_assessment` | dict               | :white_check_mark: | :x:                | :x:                |\n\nDatetime objects are created by method `datetime.fromtimestamp(timestamp)`. This means that datetime objects are in\nlocal timezone.\n\n### Documentation\n\n#### available_details\n\nReturns details which can be used to specify additional information for `identify` method. `\n\n```python\nfrom kindwise import PlantApi\n\napi = PlantApi()\n\navailable_details = api.available_details()\n```\n\n#### identify\n\nCreates a new identification. In one identification, you can include up to 5 images.\n\n```python\nimport base64\nfrom datetime import datetime\n\nfrom kindwise import PlantApi, PlantIdentification, ClassificationLevel\n\napi = PlantApi(api_key=\'your_api_key\')\n# this creates one identification composed of 5 images(not 5 different identifications)\n#\n# as input image is accepted path to an image(str / pathlib.Path), base64 encoded stream(bytes/string), stream(bytes/string),\n# or file object(supports read,seek and mode methods)\n# or PIL.Image.Image object\n# or list of images\nimages = [\'path/to/image1.jpg\', \'path/to/image2.jpg\', \'path/to/image3.jpg\', \'path/to/image4.jpg\', \'path/to/image5.jpg\']\n# details included in identification\ndetails = [\'common_names\', \'taxonomy\', \'image\']\n# disease details included in health identification(only used if health=True)\n# disease_details parameter is only available for plant.id\ndisease_details = [\'local_name\', \'description\', \'treatment\', \'cause\']\n# you can specify up to 3 languages\nlanguage = [\'en\', \'cs\']\n# default for similar_images is True\nsimilar_images = True\n# where was an image taken\nlatitude_longitude = (49.20340, 16.57318)\n# you can include health assessment in your identification by specifying health=\'all\',\n# you can also use health=\'only\' to get HealthAssessment(health assessment only)\n# health assessment is only available for plant.id\nhealth = \'all\'\n# custom id is used to identify identification in your system, but can be replaced with access_token\ncustom_id = 123\n# when was an image taken, datetime can be specified as a datetime object, timestamp, or string in ISO format\ndate_time = datetime.now()\n# default image size is 1500px, can be turned off by setting max_image_size to None\n# be aware that our API has limit 25Mpx(e.g. 5000px x 5000px)\nmax_image_size = 1500\n# you can specify into what depth should be the plant classified\n# you can choose from ClassificationLevel.SPECIES, ClassificationLevel.GENUS, ClassificationLevel.ALL\n# default is ClassificationLevel.SPECIES\nclassification_level = ClassificationLevel.SPECIES\n# in case you want to merge results for different taxon levels yourself, you can set classification_raw=True\n# be aware that the result will be in type kindwise.models.RawPlantIdentification\nclassification_raw = False\n# if our api will be ahead of this sdk and you do not want to wait for update,\n# you can specify extra_get_params or extra_post_params\nextra_get_params = None  # default\nextra_post_params = None  # default\nidentification: PlantIdentification = api.identify(\n    images,\n    details=details,\n    disease_details=disease_details,\n    language=language,\n    similar_images=similar_images,\n    latitude_longitude=latitude_longitude,\n    health=health,\n    custom_id=custom_id,\n    date_time=date_time,\n    max_image_size=max_image_size,\n    classification_level=classification_level,\n    classification_raw=classification_raw,\n    extra_get_params=extra_get_params,\n    extra_post_params=extra_post_params,\n)\n\n# identification created from stream\nwith open(\'path/to/image.jpg\', \'rb\') as image:\n    identification_from_stream: PlantIdentification = api.identify(image.read())\n\n# identification created from file object\nwith open(\'path/to/image.jpg\', \'rb\') as image:\n    identification_from_file: PlantIdentification = api.identify(image)\n\n# identification created from base64 encoded image\nwith open(\'path/to/image.jpg\', \'rb\') as image:\n    image_in_base64 = base64.b64encode(image.read())\n    identification_from_base64: PlantIdentification = api.identify(image)\n```\n\nWhen you don\'t want to wait until the identification is finished, you can specify `asynchronous=True` and\nget `access_token`  or `custom_id` if specified and retrieve the answer later.\n\n```python\nfrom kindwise import PlantApi, PlantIdentification\n\napi = PlantApi(api_key=\'your_api_key\')\n\nimage = \'path/to/image.jpg\'\nidentification: PlantIdentification = api.identify(image, asynchronous=True)\n# now you can do something else\n# ...\n# and later get identification by access_token or custom_id\nidentification: PlantIdentification = api.get_identification(identification.access_token, details=[\'common_names\'])\n```\n\n#### get_identification\n\nGet identification by token. You can specify which details you want to get. We store your identifications for 6 months.\n\n```python\nfrom kindwise import PlantApi, PlantIdentification\n\napi = PlantApi(api_key=\'your_api_key\')\n\naccess_token = \'identification_access_token\'\n# details included in identification, can be different from used in identification create\ndetails = [\'common_names\', \'taxonomy\', \'image\']\n# language can also differ from used in identification create\nlanguage = \'de\'\nidentification: PlantIdentification = api.get_identification(access_token, details=details, language=language)\n```\n\n#### delete_identification\n\nDeletes identification from our database. You can specify identification by access_token or custom_id.\n\n```python\nfrom kindwise import PlantApi\n\napi = PlantApi(api_key=\'your_api_key\')\n\ncustom_id = 123  # also works with access_token or Identification object\napi.delete_identification(custom_id)\n```\n\n#### usage_info\n\nGives you information about your api key usage.\n\n```python\nfrom kindwise import PlantApi, UsageInfo\n\napi = PlantApi(api_key=\'your_api_key\')\n\nusage_info: UsageInfo = api.usage_info()\n```\n\n#### feedback\n\nSend feedback for identification. You can specify a comment(string) or rating(int) in feedback. At least one of comment\nand rating must be specified. You can specify identification by access_token or custom_id.\n\n```python\nfrom kindwise import PlantApi\n\napi = PlantApi(api_key=\'your_api_key\')\n\ncustom_id = 123  # also works with access_token or Identification object\napi.feedback(custom_id, comment=\'comment\', rating=5)\n```\n\n#### available_disease_details\n\nReturns details which can be used to specify additional information for `health_assessment` method. Only available for\nplant.id.\n\n```python\nfrom kindwise import PlantApi\n\napi = PlantApi()\n\navailable_disease_details = api.available_disease_details()\n```\n\n#### health_assessment\n\nReturns only health assessment for identification. Health assessment is only available for plant.id. `health_assessment`\nmethod is similar to `identify` method, but it returns only health assessment. Details differs for each system.\n\n```python\nfrom datetime import datetime\n\nfrom kindwise import PlantApi, HealthAssessment\n\napi = PlantApi(api_key=\'your_api_key\')\n# the same as in identify method\nimages = [\'path/to/image1.jpg\', \'path/to/image2.jpg\', \'path/to/image3.jpg\', \'path/to/image4.jpg\', \'path/to/image5.jpg\']\n# details included in identification\ndetails = [\'local_name\', \'description\', \'treatment\', \'cause\', \'image\']\n# you can specify up to 3 languages\nlanguage = [\'en\', \'cs\']\n# default for similar_images is True\nsimilar_images = True\n# where was an image taken\nlatitude_longitude = (49.20340, 16.57318)\n# custom id is used to identify identification in your system, but can be replaced with access_token\ncustom_id = 123\n# list of suggested diseases also contains general diseases such as "Abiotic", default is False\nfull_disease_list = True\n# when was an image taken, datetime can be specified as a datetime object, timestamp, or string in ISO format\ndate_time = datetime.now()\n# default image size is 1500px, can be turned off by setting max_image_size to None\n# be aware that our API has limit 25Mpx(e.g. 5000px x 5000px)\nmax_image_size = 1500\n# if our api will be ahead of this sdk and you do not want to wait for update,\n# you can specify extra_get_params or extra_post_params\nextra_get_params = None  # default\nextra_post_params = None  # default\nidentification: HealthAssessment = api.health_assessment(\n    images,\n    details=details,\n    language=language,\n    similar_images=similar_images,\n    latitude_longitude=latitude_longitude,\n    custom_id=custom_id,\n    full_disease_list=full_disease_list,\n    date_time=date_time,\n    max_image_size=max_image_size,\n)\n```\n\nWhen you don\'t want to wait until the identification is finished, you can specify `asynchronous=True` similar\nto [`identify`](#identify) method.\n\n#### get_health_assessment\n\nGet a health assessment for identification. You can specify which details you want to get. We store your identifications\nfor 6 months.\n\n```python\nfrom kindwise import PlantApi, HealthAssessment\n\napi = PlantApi(api_key=\'your_api_key\')\n\naccess_token = \'identification_access_token\'\n# details included in identification can be different from those used in identification creation\ndetails = [\'classification\', \'local_name\']\n# language can also differ from what is used in identification creation\nlanguage = \'de\'\nfull_disease_list = False\nidentification: HealthAssessment = api.get_health_assessment(\n    access_token,\n    details=details,\n    language=language,\n    full_disease_list=full_disease_list\n)\n```\n\n#### delete_health_assessment\n\nDelete health assessment for identification.\n\n```python\nfrom kindwise import PlantApi\n\napi = PlantApi(api_key=\'your_api_key\')\n\ncustom_id = 123  # also works with access_token or HealthAssessment object\napi.delete_health_assessment(custom_id)\n```\n',
    'author': 'Simon Plhak',
    'author_email': 'simon.plhak@flowerchecker.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/flowerchecker/kindwise',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
